package com.css.simulation.resource.scene.service;

import api.common.pojo.param.scene.ScenePackageNewParam;
import api.common.pojo.param.scene.ScenePackageParam;
import api.common.pojo.param.scene.ScenePackageSublistParam;
import api.common.pojo.po.scene.ScenePackagePO;
import api.common.pojo.po.scene.ScenePackageSublistPO;
import api.common.pojo.vo.scene.ScenePackageNewVO;
import api.common.pojo.vo.scene.ScenePackageSublistVO;
import api.common.util.StringUtil;

import api.common.util.TimeUtil;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.scene.mapper.ScenePackageMapper;
import com.css.simulation.resource.scene.mapper.ScenePackageSublistMapper;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class ScenePackageService {

    @Resource
    private ScenePackageMapper scenePackageMapper;
    @Resource
    private ScenePackageSublistMapper scenePackageSublistMapper;




    /**
     * 测试包数据录入大保存：
     */
    @SneakyThrows
    public void saveScenePackage(ScenePackageNewParam params){
        ScenePackagePO po=new ScenePackagePO();
        BeanUtils.copyProperties(params,po);
        /*po.setPackageName(params.getPackageName());
        po.setWeight(params.getWeight());
        po.setShare(params.getShare());
        po.setRemarks(params.getRemarks());*/
        po.setCreateTime(TimeUtil.getNowForMysql());
        po.setModifyTime(TimeUtil.getNowForMysql());
        po.setCreateUserId(AuthUtil.getCurrentUserId());
        String uuId=StringUtil.getRandomUUID();
        po.setPackageId(uuId);
        po.setIsDeleted("0");
        po.setShare("0");
        scenePackageMapper.saveScenePackage(po);
        List<ScenePackageSublistVO> list=params.getList();
        List<ScenePackageSublistPO>  poList=new ArrayList<>();
        List<ScenePackageSublistPO> treeList =instShareListTree(list,uuId,uuId,poList,"0");
        scenePackageSublistMapper.saveScenePackageSublistAll(treeList);
    }
    //查询测试包主表列表
    @SneakyThrows
    public   List<ScenePackagePO> queryScenePackageList(ScenePackageParam params){
        List<ScenePackagePO> list = scenePackageMapper.queryScenePackageList(params);
        return list;
    }

    //修改主表数据
    @SneakyThrows
    public   void updateScenePackage(ScenePackageNewParam params) {
        ScenePackagePO po=new ScenePackagePO();
        BeanUtils.copyProperties(params,po);
        po.setCreateTime(TimeUtil.getNowForMysql());
        po.setModifyTime(TimeUtil.getNowForMysql());
        po.setCreateUserId(AuthUtil.getCurrentUserId());
        po.setIsDeleted("0");
        po.setShare("0");
        scenePackageMapper.updateScenePackage(po);
        scenePackageSublistMapper.deleteScenePackageSublist(params.getPackageId());
        List<ScenePackageSublistVO> list=params.getList();
        List<ScenePackageSublistPO>  poList=new ArrayList<>();
        List<ScenePackageSublistPO> treeList =instShareListTree(list,params.getPackageId(),params.getPackageId(),poList,"0");
        scenePackageSublistMapper.saveScenePackageSublistAll(treeList);
    }

    //删除测试包
    @SneakyThrows
    public   void deleteList(ScenePackageParam params) {
        ScenePackagePO po=new ScenePackagePO();
        po.setModifyUserId(StringUtil.getRandomUUID());
        po.setModifyTime(TimeUtil.getNowForMysql());
        po.setIsDeleted("1");
        po.setPackageId(params.getPackageId());
        scenePackageSublistMapper.deleteScenePackageSublist(params.getPackageId());
        scenePackageMapper.deleteScenePackage(po);
    }

    //查询一个测试包的指标列表
    @SneakyThrows
    public ScenePackageNewVO queryScenePackageSublistList(ScenePackageParam params){
        ScenePackageSublistPO po=new ScenePackageSublistPO();
        po.setParentId(params.getPackageId());
        ScenePackageNewVO vo= scenePackageMapper.queryScenePackageBySublist(params.getPackageId());
        List<ScenePackageSublistVO> list =selectScenePackageSubListTree(null,true,params.getPackageId());
        vo.setList(list);
     //   vo.setList(list);
        return vo;
    }

    /**
     * 级联获取场景指标
     * @param parentVoList 首次传null
     * @param isRoot 首次传true
     * @param scenePackageId 场景包id
     * @return
     */
    public List<ScenePackageSublistVO> selectScenePackageSubListTree(List<ScenePackageSublistVO> parentVoList,
                                                                     boolean isRoot,String scenePackageId){
        ScenePackageSublistPO params=new ScenePackageSublistPO();
        params.setParentId(scenePackageId);
        if(isRoot){
            //查找一级节点
            parentVoList = scenePackageSublistMapper.queryScenePackageSublistList(params);
            if(parentVoList != null && parentVoList.size() > 0){
                selectScenePackageSubListTree(parentVoList, false, null);
            }
        }else{
            //获取子节点集合
            for(ScenePackageSublistVO pvo : parentVoList){
                params.setParentId(pvo.getSublistId());
                List<ScenePackageSublistVO> cvoList = scenePackageSublistMapper.queryScenePackageSublistList(params);
                if(cvoList != null && cvoList.size() > 0){
                    //存入父节点集合中
                    pvo.setList(cvoList);
                    pvo.setHasChildren(true);
                    //继续查找下一节点
                    selectScenePackageSubListTree(cvoList,false, null);
                }
            }
        }
        return parentVoList;
    }


//分享
    @SneakyThrows
    public void instShareList(ScenePackageParam params){
        //查询测试包主表
        ScenePackagePO scenePackagePO=scenePackageMapper.queryScenePackage(params);
        ScenePackagePO po=new ScenePackagePO();
        BeanUtils.copyProperties(scenePackagePO,po);
        po.setCreateTime(TimeUtil.getNowForMysql());
        po.setModifyTime(TimeUtil.getNowForMysql());
        po.setCreateUserId(AuthUtil.getCurrentUserId());
        String uuId=StringUtil.getRandomUUID();
        po.setPackageId(uuId);
        po.setIsDeleted("0");
        po.setShare("1");
        scenePackageMapper.saveScenePackage(po);
        List<ScenePackageSublistPO>  poList=new ArrayList<>();
        List<ScenePackageSublistVO> listSublist =selectScenePackageSubListTree(null,true,params.getPackageId());
        List<ScenePackageSublistPO> treeList =instShareListTree(listSublist,uuId,uuId,poList,"1");
       scenePackageSublistMapper.saveScenePackageSublistAll(treeList);
    }
    /**
     * 根据级联结果转化为单级list
     * @param parentVoList 首次传null
     * @param scenePackageId 场景包id
     * @return
     */
    public List<ScenePackageSublistPO> instShareListTree(List<ScenePackageSublistVO> parentVoList,
                                                                    String scenePackageId,String rootId,List<ScenePackageSublistPO>  poList,String share){
        {
            //获取子节点集合
                for (int i=0;i< parentVoList.size() ;i++) {
                ScenePackageSublistPO params=new ScenePackageSublistPO();
                BeanUtils.copyProperties(parentVoList.get(i),params);
                params.setParentId(scenePackageId);
                params.setRootId(rootId);
                params.setSublistName(parentVoList.get(i).getSublistName());
                params.setCreateTime(TimeUtil.getNowForMysql());
                params.setModifyTime(TimeUtil.getNowForMysql());
                params.setCreateUserId(AuthUtil.getCurrentUserId());
                String uuId= StringUtil.getRandomUUID();
                params.setSublistId(uuId);   //主键ID
                params.setIsDeleted("0");
                params.setShare(share);
                List<ScenePackageSublistVO> cvoList = parentVoList.get(i).getList();
                poList.add(params);
                if(cvoList != null && cvoList.size() > 0){
                    //继续查找下一节点
                    instShareListTree(cvoList, uuId,rootId,poList,share);
                }
                parentVoList.remove(parentVoList.get(i));
                i--;
            }
        }
        return poList;
    }


}
